/*
  "mtime" event test: Event trigger test which operates on the static
  event data items "start", "end", "interval" only.

  10/2017: F. Prill (DWD)
*/

/*
  TODO #1:

  - check if event interval is positive.
  - check if end date is larger than start date.
  - correct interval-to-msecs computation for other calendar types.
  - include error handling in this routine.

  TODO #2:
  
  - The regexp for newDateTime does not accept leading spaces!
  - A public interface for "elementwiseScalarMultiplyTimeDeltaDP" is missing.
  - "elementwiseScalarMultiplyTimeDeltaDP": why must output "scaled_td" be pre-allocated?
  - "addTimeDeltaToDateTime": why must output dateTime be pre-allocated?
  - camel case is inconsistent: "compareDatetime" but "constructAndCopyDateTime".
*/

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

#include "mtime_calendar.h"
#include "mtime_datetime.h"
#include "mtime_julianDay.h"
#include "mtime_timedelta.h"
#include "mtime_eventHandling.h"

#define _GET_X86_COUNTER 1

#ifdef _GET_X86_COUNTER
#include <x86intrin.h>
#endif

/*
  Tests if event has been triggered.
  
  This routine locates the event period where "query date" lies in and
  returns if the event has been triggered. Note that we operate on the
  static event data items "start", "end", "interval" only.
 */
  
int main(int argc, char *argv[])
{
#ifdef _GET_X86_COUNTER
  uint64_t start_old_active_test, stop_old_active_test;  
  uint64_t start_new_active_test, stop_new_active_test;
#endif

  bool old_triggered, new_triggered, fail_triggered;

  char *eString = (char *) calloc(MAX_EVENT_STR_LEN, sizeof(char));
  
  printf("Event test\n----------\n\n");
  initCalendar(PROLEPTIC_GREGORIAN); 

  struct _event *e_fail = newEvent("Event: failing", "2013-01-01T00:00:00.000", "2013-01-01T00:00:00.000", "2019-01-01T00:00:00.000", "P1M", NULL);

  struct _datetime  *query_date = newDateTime("2013-01-01T00:00:00.000");
  fail_triggered = isCurrentEventActive(e_fail, query_date, NULL, NULL);

  free(query_date);

  exit(1);
    
  /* --- test data: example event (start,end,interval): */
  struct _event *e_old = newEvent("Event: old scheme", "888-09-01T02:10:00.000", "888-09-01T02:10:00.000", "2013-09-03T14:00:00.000", "P1M", NULL);
  struct _event *e_new = newEvent("Event: new scheme", "888-09-01T02:10:00.000", "888-09-01T02:10:00.000", "2013-09-03T14:00:00.000", "P1M", NULL);
  
  /* --- test data: example "is-event-active" query: */
  query_date = newDateTime("1000-09-01T02:10:00.000");
  
  eString = eventToString(e_old, eString);  
  fprintf(stderr, "%s", eString);
  eString = eventToString(e_new, eString);
  fprintf(stderr, "%s", eString);
  
#ifdef _GET_X86_COUNTER
  start_new_active_test = _rdtsc();
#endif
  new_triggered = isCurrentEventActive(e_new, query_date, NULL, NULL);
#ifdef _GET_X86_COUNTER
  stop_new_active_test = _rdtsc();
  printf("New event active test: %" PRIu64 "\n",  stop_new_active_test - start_new_active_test);
#endif
  if (new_triggered)
    {
      printf("EVENT TRIGGERED!\n\n");
    }

#ifdef _GET_X86_COUNTER
  start_old_active_test = _rdtsc();
#endif
  old_triggered = isCurrentEventActive_old(e_old, query_date, NULL, NULL);
#ifdef _GET_X86_COUNTER
  stop_old_active_test = _rdtsc();
  printf("Old event active test: %" PRIu64 "\n",  stop_old_active_test - start_old_active_test);
#endif
  if (old_triggered)
    {
      printf("EVENT TRIGGERED!\n\n");
    }

  eString = eventToString(e_old, eString);  
  fprintf(stderr, "%s", eString);
  eString = eventToString(e_new, eString);
  fprintf(stderr, "%s", eString);

  free(eString);
  
  deallocateDateTime(query_date);
  deallocateEvent(e_old);
  deallocateEvent(e_new);
  
  return 0;
}
